@@ -536,10 +536,6 @@ DESCRIPTION
 .#define bfd_mach_aarch64_8R	1
 .#define bfd_mach_aarch64_ilp32	32
 .#define bfd_mach_aarch64_llp64 64
-.  bfd_arch_nios2,     {* Nios II.  *}
-.#define bfd_mach_nios2		0
-.#define bfd_mach_nios2r1	1
-.#define bfd_mach_nios2r2	2
 .  bfd_arch_visium,    {* Visium.  *}
 .#define bfd_mach_visium	1
 .  bfd_arch_wasm32,    {* WebAssembly.  *}
@@ -571,9 +567,8 @@ DESCRIPTION
 .#define bfd_mach_amdgcn_gfx908          0x030
 .#define bfd_mach_amdgcn_gfx90a          0x03f
 .#define bfd_mach_amdgcn_gfx9_4_generic  0x05f
-.#define bfd_mach_amdgcn_gfx940          0x040
-.#define bfd_mach_amdgcn_gfx941          0x04b
 .#define bfd_mach_amdgcn_gfx942          0x04c
+.#define bfd_mach_amdgcn_gfx950          0x04f
 .#define bfd_mach_amdgcn_gfx10_1_generic 0x052
 .#define bfd_mach_amdgcn_gfx1010         0x033
 .#define bfd_mach_amdgcn_gfx1011         0x034
@@ -586,6 +581,8 @@ DESCRIPTION
 .#define bfd_mach_amdgcn_gfx1100         0x041
 .#define bfd_mach_amdgcn_gfx1101         0x046
 .#define bfd_mach_amdgcn_gfx1102         0x047
+.#define bfd_mach_amdgcn_gfx1150         0x043
+.#define bfd_mach_amdgcn_gfx1151         0x04a
 .#define bfd_mach_amdgcn_gfx12_generic   0x059
 .#define bfd_mach_amdgcn_gfx1200         0x048
 .#define bfd_mach_amdgcn_gfx1201         0x04e
@@ -692,7 +689,6 @@ extern const bfd_arch_info_type bfd_msp430_arch;
 extern const bfd_arch_info_type bfd_mt_arch;
 extern const bfd_arch_info_type bfd_nds32_arch;
 extern const bfd_arch_info_type bfd_nfp_arch;
-extern const bfd_arch_info_type bfd_nios2_arch;
 extern const bfd_arch_info_type bfd_ns32k_arch;
 extern const bfd_arch_info_type bfd_or1k_arch;
 extern const bfd_arch_info_type bfd_pdp11_arch;
@@ -781,7 +777,6 @@ static const bfd_arch_info_type * const bfd_archures_list[] =
     &bfd_mt_arch,
     &bfd_nds32_arch,
     &bfd_nfp_arch,
-    &bfd_nios2_arch,
     &bfd_ns32k_arch,
     &bfd_or1k_arch,
     &bfd_pdp11_arch,
@@ -958,6 +953,7 @@ bfd_arch_get_compatible (const bfd *abfd,
      to assume that they know what they are doing.  */
   if (accept_unknowns
       || ubfd->plugin_format == bfd_plugin_yes
+      || ubfd->plugin_format == bfd_plugin_yes_unused
       || strcmp (bfd_get_target (ubfd), "binary") == 0)
     return kbfd->arch_info;
   return NULL;
