@@ -18,8 +18,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#if !defined (DWARF2LOC_H)
-#define DWARF2LOC_H
+#ifndef GDB_DWARF2_LOC_H
+#define GDB_DWARF2_LOC_H
 
 #include "dwarf2/expr.h"
 
@@ -198,6 +198,9 @@ struct dwarf2_loclist_baton
   /* Non-zero if the location list lives in .debug_loc.dwo.
      The format of entries in this section are different.  */
   unsigned char from_dwo;
+
+  /* The version of DWARF this loclist comes from.  */
+  unsigned char dwarf_version;
 };
 
 /* The baton used when a dynamic property is an offset to a parent
@@ -309,4 +312,4 @@ extern struct value *value_of_dwarf_reg_entry (struct type *type,
 					       const frame_info_ptr &frame,
 					       enum call_site_parameter_kind kind,
 					       union call_site_parameter_u kind_u);
-#endif /* DWARF2LOC_H */
+#endif /* GDB_DWARF2_LOC_H */
