@@ -17,8 +17,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#if !defined (GCORE_H)
-#define GCORE_H 1
+#ifndef GDB_GCORE_H
+#define GDB_GCORE_H
 
 #include "gdb_bfd.h"
 
@@ -36,4 +36,4 @@ extern int objfile_find_memory_regions (find_memory_region_ftype func,
 
 extern thread_info *gcore_find_signalled_thread ();
 
-#endif /* GCORE_H */
+#endif /* GDB_GCORE_H */
