@@ -20,8 +20,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef MI_MI_PARSE_H
-#define MI_MI_PARSE_H
+#ifndef GDB_MI_MI_PARSE_H
+#define GDB_MI_MI_PARSE_H
 
 #include "gdbsupport/run-time-clock.h"
 #include <chrono>
@@ -106,4 +106,4 @@ private:
 
 enum print_values mi_parse_print_values (const char *name);
 
-#endif /* MI_MI_PARSE_H */
+#endif /* GDB_MI_MI_PARSE_H */
